/*
###############################################################################
# This file is distributed as part of TICKET, but was written independently.
# It is Copyright (C) 2001 by Eric H. Weigle.
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# ------------------------------------------------------------
# 
# You may contact the author (Eric H. Weigle) via email at weigle@cs.wisc.edu,
# or via postal mail at MS D451, P.O. Box 1663, Los Alamos NM  87545.
# 
# Should you fail to reach the author at these addresses, please refer to the
# following website for updated information or other means by which he can
# be contacted:
# 	http://www.losalamos.com/ehw/
#
###############################################################################
*/
// #############################################################################
// Doubly linked list header file.
// #############################################################################

#ifndef EW_DOUB_LINK_LIST_H
#define EW_DOUB_LINK_LIST_H

#include <stdio.h>

// ----------------------------------------
// A single list item.
// ----------------------------------------
struct dlist_item {
	void *data;
	struct dlist_item *next, *prev;
};

// ----------------------------------------
// The whole list.
// ----------------------------------------
struct dlinked_list {
	struct dlist_item *head, *tail;
	int count;

	// semantics of memcmp (<0, 0, >0)
	int (*compfn)(void *item_a, void *item_b);
	// just to handle printing out 
	void (*printfn)(FILE *file, void *item);
};

// ----------------------------------------
// Insert/Add functions.
// ----------------------------------------
// make new item the head.
void dlist_push_head(struct dlinked_list *list, void *item);
// make new item the tail.
void dlist_push_tail(struct dlinked_list *list, void *item);
// insertion sort.
void dlist_insert(struct dlinked_list *list, void *item);

// ----------------------------------------
// Removal functions.
// ----------------------------------------
// chop off the head.
int  dlist_pop_head(struct dlinked_list *list, void **item);
// chop off the tail.
int  dlist_pop_tail(struct dlinked_list *list, void **item);
// remove first instance 'item' from list, if it exists.
int dlist_remove(struct dlinked_list *list, void *item);

// ----------------------------------------
// Other functions.
// ----------------------------------------
// print the list on the given file.
void dlist_print(struct dlinked_list *list, FILE *file);
// check that everything's correct.
int dlist_verify(struct dlinked_list *list);

#endif

