/*
###############################################################################
# This file is part of TICKET, the Traffic Information Collection Kernel with
# Exact Timing. This program and all associated files are Copyright (C) 2001
# by Eric H. Weigle of Los Alamos National Laboratory.
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# ------------------------------------------------------------
# 
# You may contact the author (Eric H. Weigle) via email at ehw@lanl.gov,
# or via postal mail at MS D451, P.O. Box 1663, Los Alamos NM  87545.
# 
# Should you fail to reach the author at these addresses, please refer to the
# following websites for updated information or other means by which he can
# be contacted:
# 	http://public.lanl.gov/ehw/ 
# 	http://public.lanl.gov/radiant/
#
###############################################################################
*/
// #############################################################################
// This file takes care of differences between user and kernel level code.
// In particular, code compiled at both user and kernel level has several
// somewhat arbitrary but simple changes, such as printf<-->printk, etc.
// We account for these with some #defines.
// #############################################################################

#ifndef TICKET_USERKERNEL_H
#define TICKET_USERKERNEL_H

#ifdef __KERNEL__
// =============================================================================
// if at kernel level, then set up kernel stuff.
// =============================================================================

#define ALLOC_FUNCTION kmalloc
#define ALLOC_PRIORITY , GFP_KERNEL
#define FREE_FUNCTION kfree
#define PRINT_FUNCTION printk
#define PRIORITY_NORMAL KERN_NOTICE
#define PRIORITY_ERROR KERN_EMERG
void ew_panic(const char *msg);

#else
// =============================================================================
// If at user level, then set up user stuff.
// =============================================================================

#define ALLOC_FUNCTION malloc
#define ALLOC_PRIORITY /* none for user level, priority for kernel */
#define FREE_FUNCTION free
#define PRINT_FUNCTION fprintf
#define PRIORITY_NORMAL stdout,
#define PRIORITY_ERROR stderr,
// This is an actual function in the kernel, here just print and die.
#define ew_panic(x) fprintf(stderr, x); exit(1)

#endif

#endif
