/*
###############################################################################
# This file is part of TICKET, the Traffic Information Collection Kernel with
# Exact Timing. This program and all associated files are Copyright (C) 2001
# by Eric H. Weigle of Los Alamos National Laboratory.
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# ------------------------------------------------------------
# 
# You may contact the author (Eric H. Weigle) via email at ehw@lanl.gov,
# or via postal mail at MS D451, P.O. Box 1663, Los Alamos NM  87545.
# 
# Should you fail to reach the author at these addresses, please refer to the
# following websites for updated information or other means by which he can
# be contacted:
# 	http://public.lanl.gov/ehw/ 
# 	http://public.lanl.gov/radiant/
#
###############################################################################
*/
// #############################################################################
// In certain cases I cannot do NOT do the DNS lookup or ARP request.
// This is because TICKET is *completely* passive, I can't get its MAC address
// without knowing it before-hand (it doesn't reply to arp who-has messages).
//
// Thus I use a pre-built table for the kernel code and the 'killer' code.
//
// For the IPs, using a string is not the fastest way to do it,
// but it's the most legible in my opinion. (It'd be faster to
// precompute the IPs in NBO as and do a direct hash to those values).
//
// Same thing with the MAC addresses.
//
// By the fact that this is in the 'common' directory you realize that it is
// used by both kernel and user-level code. Trust me, this is the easiest and
// most reliable way to do it.
// #############################################################################

#include "config.h"

#ifndef TICKET_ADDRESSES_H
#define TICKET_ADDRESSES_H

// This define is also found in the user-level header "/usr/include/linux/if_ether.h"
// The enthernet address length (bytes)
#ifndef ETH_ALEN
#define ETH_ALEN 6
#endif

// =============================================================================
// Structures
// =============================================================================

// holds the mapping in the format above, easy to edit/modify.
struct name_mapping_raw {
	char *name;
	char *ip;
	char *mac;
};

// This holds the mapping after getting it usable.
struct name_mapping {
	char *name;
	unsigned int ip; /* network byte order */
	unsigned char mac[ETH_ALEN];
};


// =============================================================================
// Functions
// =============================================================================
int hextodec(char c);
char *mac2str(const unsigned char *macaddr, char *macstr);
char *ip2str_hbo(const unsigned int ipaddr, unsigned char *ipstr);
char *ip2str_nbo(const unsigned int ipaddr, unsigned char *ipstr);
int str2mac(char *str, unsigned char mac[ETH_ALEN]);
unsigned int str2ip(char *str, unsigned int *ip);
struct name_mapping *process_mapping(struct name_mapping_raw *rawnames);
void free_mapping(struct name_mapping *processednames);
struct name_mapping *get_addr_info(struct name_mapping *table, char *name, unsigned int ip, unsigned char *mac);
void print_mapping(struct name_mapping *map);

#endif

