/*
###############################################################################
# This file is part of TICKET, the Traffic Information Collection Kernel with
# Exact Timing. This program and all associated files are Copyright (C) 2001
# by Eric H. Weigle of Los Alamos National Laboratory.
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# ------------------------------------------------------------
# 
# You may contact the author (Eric H. Weigle) via email at ehw@lanl.gov,
# or via postal mail at MS D451, P.O. Box 1663, Los Alamos NM  87545.
# 
# Should you fail to reach the author at these addresses, please refer to the
# following websites for updated information or other means by which he can
# be contacted:
# 	http://public.lanl.gov/ehw/ 
# 	http://public.lanl.gov/radiant/
#
###############################################################################
*/
// #############################################################################
// Dump header information, shared between user and kernel programs:
//    kernel/dump.c, user{reader, writer, printer}
//
// Gives info on collected information, record sizes, etc.
// #############################################################################

#ifndef TICKET_COMMON_INFO
#define TICKET_COMMON_INFO

#include "shared_network.h"

// =============================================================================
// Defines of useful stuff
// =============================================================================

// ABSOLUTE dump header length
#define DUMP_HEADER_LEN 8

// Maximum size of packet collected off the wire
#define SNAPLEN (ETH_HEADER_LEN+ETH_DATA_LEN)

// how big our dump record is.
#define DUMP_RECORD_LEN (sizeof(struct collected_headers))

// Total overhead of headers in a dump packet.
#define DUMP_OVERHEAD_LEN (ETH_HEADER_LEN+IP_HEADER_LEN+UDP_HEADER_LEN+DUMP_HEADER_LEN)

// Maximum space available to our collected headers.
#define MAX_DUMP_LEN (SNAPLEN-DUMP_OVERHEAD_LEN)

// (1500-overheads/DUMP_RECORD_LEN) = 34 (total 1470+8TS == 22 bytes free)
#define DUMPS_PER_PACKET (MAX_DUMP_LEN/DUMP_RECORD_LEN)

// Amount of space left for alignment, etc.
#define LEFTOVER_SPACE (MAX_DUMP_LEN - (DUMPS_PER_PACKET*DUMP_RECORD_LEN))


// =============================================================================
// The actual stuff we're collecting.
//
// TODO: Version with just 68 byte character array; selected via DUMP_OUTPUT_FORMAT
//       option in config file. (see common/config.h)
// =============================================================================
struct collected_headers {
	// Basic information.
	unsigned long long timestamp; // cycle counters
	unsigned short length;        // 'off-the-wire' length (in bytes, up to 1514<2048=2^11)

	// Ethernet/802.3
	unsigned char mac_dst[ETH_ALEN]; // ethernet
	unsigned char mac_src[ETH_ALEN]; // ethernet
	unsigned short toslen;    // MAC TOS/802.3 length (usually 0x0800==IP)

	// IP information
	unsigned int ip_src;      // IP source
	unsigned int ip_dst;      // IP destination
	unsigned short ip_len;    // IP packet length
	unsigned char ip_hdr_len; // IP header length
	unsigned char protocol;   // IP protocol field

	// TCP/UDP information
	unsigned short src_port;  // TCP/UDP source port
	unsigned short dst_port;  // TCP/UDP destination port
	union {
		unsigned short udp_len; // UDP length field (in bytes)
		// len must come BEFORE pad to print
		// correctly as short on little-endian box.
		struct {
			unsigned char len;  // TCP header length field (in 32-bit words)
			unsigned char pad;  // empty; ignore!
		} tcp_len;
	} upper_len;
} __attribute__ ((packed));

struct dump_packet {
	unsigned long long seqno;
	struct collected_headers data[DUMPS_PER_PACKET];
};

#endif

