/*
###############################################################################
# This file is part of TICKET, the Traffic Information Collection Kernel with
# Exact Timing. This program and all associated files are Copyright (C) 2001
# by Eric H. Weigle of Los Alamos National Laboratory.
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# ------------------------------------------------------------
# 
# You may contact the author (Eric H. Weigle) via email at ehw@lanl.gov,
# or via postal mail at MS D451, P.O. Box 1663, Los Alamos NM  87545.
# 
# Should you fail to reach the author at these addresses, please refer to the
# following websites for updated information or other means by which he can
# be contacted:
# 	http://public.lanl.gov/ehw/ 
# 	http://public.lanl.gov/radiant/
#
###############################################################################
*/
// #############################################################################
// Miscellaneous user functions.
//
// At the moment, just get_timestamp and get_cpuinfo, for timing info.
// #############################################################################

#include "user_misc.h"
#include <stdio.h>  // for FILE
#include <string.h> // for strstr


// =============================================================================
// Get the long long timestamp out of the cpu cycle-counter register
// I use this all over the place as a high-resolution way of timing things.
// =============================================================================
unsigned long long get_timestamp(void) {
	register unsigned int dummylo;
	register unsigned int dummyhi;
	unsigned long long dummy;

	asm volatile (".byte 0x0f, 0xa2" : : : "eax", "ebx", "ecx", "edx");
	asm volatile (".byte 0x0f, 0x31;              # the RDTSC instruction
					movl %%eax, %0
					movl %%edx, %1"
					: "=r" (dummylo), "=r" (dummyhi)
					:
					: "eax", "edx"
					);

	dummy = dummyhi;
	dummy = (dummy << 32) | dummylo;

	return dummy;
}


// =============================================================================
// This function makes all sorts of assumptions about the format of
//     /proc/cpuinfo, which undoubtedly will change dramatically all the time.
// If it fails, you probably want to just replace this function with a stub
//     stub that returns the proper number.
// =============================================================================
int get_cpuinfo(void) {
	FILE *file;
	char line[1024], *idx;
	int nread;
	double speed;

	// open the cpu informatino file from /proc
	// and just read in the whole darn thing.
	if ((file = fopen("/proc/cpuinfo", "r"))==NULL) {
		fprintf(stderr, "Failed to open /proc/cpuinfo\n");
		return 0;
	}
	if ((nread = fread(line, 1, 1024, file))<0) {
		fprintf(stderr, "Failed to read /proc/cpuinfo\n");
		return 0;
	}
	fclose(file);

	// find the part of it giving the cpu's MHZ,
	// then skip past the colon to find the actual number.
	// The line looks like "cpu MHz         : 731.551"
	if ((idx = strstr(line, "cpu MHz"))==NULL) {
		fprintf(stderr, "Failed to find cpu speed 1\n");
		return 0;
	}
	if ((idx = strstr(idx, ": "))==NULL) {
		fprintf(stderr, "Failed to find cpu speed 2\n");
		return 0;
	}

	// scan in the number, it's in MHz with 3 decimal point
	// precision (giving an effective kHz measure)
	sscanf(idx+2, "%lf", &speed);
	if ((idx = strstr(idx, "flags"))==NULL) {
		fprintf(stderr, "Failed to find cpu flags\n");
		return 0;
	}

	// make sure that we can actually use rdtsc; if we can
	// then there's a flag 'tsc' in the 'flags' field.
	if ((idx = strstr(idx, "tsc"))==NULL) {
		printf("CPU DOES NOT appear to support reliable TSC\n");
		return 0;
	}

	// Ok! Convert to Hz from MHz.
	return ((int)(speed*1000*1000));
}

