/*
###############################################################################
# This file is part of TICKET, the Traffic Information Collection Kernel with
# Exact Timing. This program and all associated files are Copyright (C) 2001
# by Eric H. Weigle of Los Alamos National Laboratory.
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# ------------------------------------------------------------
# 
# You may contact the author (Eric H. Weigle) via email at ehw@lanl.gov,
# or via postal mail at MS D451, P.O. Box 1663, Los Alamos NM  87545.
# 
# Should you fail to reach the author at these addresses, please refer to the
# following websites for updated information or other means by which he can
# be contacted:
# 	http://public.lanl.gov/ehw/ 
# 	http://public.lanl.gov/radiant/
#
###############################################################################
*/
// #############################################################################
// (Yet Another) ``Flooder'' program:
//     sends UDP packets as fast as possible to a given host.
//
//     This is my flooder. There are many like it. But this one is mine. :)
// #############################################################################
#include "../common/user_network.h"
#include "../common/user_misc.h"
#include <signal.h>
#include <stdlib.h>
#include <unistd.h>


// ============================================================================
// Local defines. Could be configuration parameters, but they seemed trivial
// enough that it didn't seem worth the trouble of configuring it that way.
// ============================================================================
// the packet size we send
#define MAX_MESSAGE_SIZE 1024
// the actual message size 'on-the-wire' for calculating how fast we're sending.
#define ABSOLUTE_MESSAGE_SIZE (MAX_MESSAGE_SIZE+UDP_HEADER_LEN+IP_HEADER_LEN+ETH_HEADER_LEN)

// clearer way of writing whether I'm currently running
// as a client (sending) or server (receiving).
#define FLOODER_MODE_CLIENT 1
#define FLOODER_MODE_SERVER 2


// ============================================================================
//  GLOBALS
// ============================================================================
// program configuration
int mode;
int verbosity=0;

// Network stuff
char *hostname;    // client's server name
struct sockaddr_in sock_addr; // send/recv socket address
int connfd=0;      // send/recv socket descriptor
long packet_count; // count of packets sent/recv'd
int bufsize;       // units in bytes

// Timing issues
int local_cpu_hz;  // local cpu's cycles per second :)

// packet buffer
char *msg=NULL;



// ============================================================================
// FUNCTION DECLARATIONS
// ============================================================================
void flooder_shutdown(int sig);
int flooder_startup(int send_duration, int send_rate);
void flood_send(int rate, int duration);
void flood_recv(int passive);
void usage(char *error);
void parse_command_line(int argc, char *argv[], int *send_duration, int *send_rate, int *passive);



// ============================================================================
// Program shutdown routine.
// Called either as signal handler, or directly when we're done.
// ============================================================================
void flooder_shutdown(int sig) {
	if (verbosity>0) {
		printf("%s shutting down.\n",
				(mode==FLOODER_MODE_CLIENT ? "Client" : "Server"));
	}
	if (connfd!=0) { close(connfd); }
	if (msg!=NULL) { free(msg); }

	exit(0);
}


// ============================================================================
// Program startup routine.
// Called immediately after parsing command-line.
// Initializes memory, configures network, sets up proper termination.
// ============================================================================
int flooder_startup(int send_duration, int send_rate) {
	struct sigaction my_sigact;

	// get CPU speed;
	if ((local_cpu_hz = (int)get_cpuinfo())==0) {
		printf("Failed to get cpu info!\n");
		return 0;
	}

	// reserve space for our messages
	if ((msg = (char *)malloc(MAX_MESSAGE_SIZE))==NULL) {
		printf("Could not allocate message packet.\n");
		return 0;
	}
	bzero((char *)msg, MAX_MESSAGE_SIZE);

	// set up termination.
	my_sigact.sa_flags = 0x0; // nothing special
	my_sigact.sa_sigaction = NULL; // no function to handle extra info
	my_sigact.sa_handler = &(flooder_shutdown); // signal handler function
	if (sigaction(SIGINT, &my_sigact, NULL)) {
		perror("Could not set up termination handling\n");
		return 0;
	}

	// set up network stuff.
	if ((connfd=create_udp_socket((mode==FLOODER_MODE_CLIENT ? hostname : NULL), &sock_addr, bufsize, verbosity))==0) {
		printf("Failed to set up UDP network 'connection'\n");
		return 0;
	}

	// notify user of progress.
	if (verbosity > 0) {
		if (mode==FLOODER_MODE_CLIENT) {
			printf("Client started up on %d Hz host. Sending at %dMbps for %d seconds.\n", local_cpu_hz, send_rate, send_duration);
		} else {
			printf("Server started up on %d Hz host. Awaiting client messages.\n", local_cpu_hz);
		}
	}

	return 1;
}



// ============================================================================
// send at 'rate' megabits per second, for 'duration' seconds.
//
// Set up spacing (milliseconds or cycles per packet).
//     1. 1000/(packets per second) = milliseconds per packet.
//     2. (packets per second) = ((rate(mb) * 1,000,000)/ (8 bits/byte))/(sizeof(packet)=MAX_MESSAGE_SIZE).
//     3. (via 1, 2). (1000)/(((rate*1,000,000)/8)/MAX_MESSAGE_SIZE) = milliseconds/packet
//     4. (via 3). (1000*8*MAX_MESSAGE_SIZE)/(rate*1,000,000) = milliseconds/packet
//     5. (via 4). (MAX_MESSAGE_SIZE)/(rate*125) = milliseconds/packet
// Or
//     1. (local_cpu_hz)/(packets per second) = local_cpu_hz per packet.
//     2. (packets per second) = ((rate(mb) * 1,000,000)/ (8 bits/byte))/(sizeof(packet)=MAX_MESSAGE_SIZE).
//     3. (via 1, 2). (local_cpu_hz)/(((rate*1,000,000)/8)/MAX_MESSAGE_SIZE) = local_cpu_hz per packet.
//     4. (via 3). (local_cpu_hz*8*MAX_MESSAGE_SIZE)/(rate*1,000,000) = local_cpu_hz/packet
//     5. (via 4). (local_cpu_hz*MAX_MESSAGE_SIZE)/(rate*125,000) = local_cpu_hz/packet
// ============================================================================
void flood_send(int rate, int duration) {
	unsigned long long sleep_time_cycles;
	unsigned long long now_and_then, display_time, last_display_time;
	int burst_length, idx, jdx;
	// Timing issues
	unsigned long long start_time, stop_time; // units in cycles
	unsigned long long base_start_time, total_packet_count;
	double sleep_time_ms;

	// first, calculate how long in milliseconds between packets.
	// If sleep_time_ms is below 10ms, then send bursts (maximal precision is about 10ms).
	sleep_time_ms = ((double)(ABSOLUTE_MESSAGE_SIZE))/((double)rate*125);
	if (sleep_time_ms<10) {
		burst_length = (int)(10.0 / sleep_time_ms)+1;
	} else {
		burst_length=1;
	}
	sleep_time_ms = burst_length*sleep_time_ms;
	sleep_time_cycles = (unsigned long long)(local_cpu_hz/1000 * sleep_time_ms);

	// Initialize timing.
	start_time = get_timestamp();
	base_start_time = start_time;
	stop_time = start_time + ((unsigned long long)duration)*local_cpu_hz;
	last_display_time = start_time;
	display_time = last_display_time+local_cpu_hz; // give a message every second.

	if (verbosity>1) {
		printf("Sending at rate %dMbps, that's %f ms/packet\n", rate, sleep_time_ms);
		printf("Bursting %d packets to create rate of %dMbps, that's %f ms/burst, or %lld cycles/burst\n", burst_length, rate, sleep_time_ms, sleep_time_cycles);
		printf("duration=%d,%d,%lld, %d\n", duration, duration/1000, (unsigned long long)(duration/1000)*local_cpu_hz, local_cpu_hz);
		printf("  Start time: %lld\n", start_time);
		printf("display_time: %lld\n", display_time);
		printf("   Stop_time: %lld\n", stop_time);
		printf("Target rate %d packets sent \n", rate*1000000);
	}

	// loop while time left to send...
	total_packet_count=0;
	packet_count=0;
	for (idx=0; get_timestamp()<stop_time; idx++) {
		// send a burst
		for (jdx=0; jdx<burst_length; jdx++) {
			// Send a packet to the server.
			if (!full_sendto(connfd, msg, MAX_MESSAGE_SIZE, &sock_addr, "ping message")) {
				printf("Failure in rate_send. Quitting.\n");
				return;
			}
			packet_count++;
		}

		// every second report rates.
		now_and_then=get_timestamp();
		if (verbosity>0 && now_and_then>=display_time) {
			total_packet_count += packet_count;

			printf("Observed rate %9.3f over %ld packets, historically (%0.3f seconds) rate %9.3f over %lld packets\n",
					((double)((packet_count*(ABSOLUTE_MESSAGE_SIZE)*8)/(((double)(now_and_then-last_display_time))/local_cpu_hz)))/1000000,
					packet_count,
					(((double)(now_and_then-base_start_time))/local_cpu_hz),
					((double)((total_packet_count*(ABSOLUTE_MESSAGE_SIZE)*8)/(((double)(now_and_then-base_start_time))/local_cpu_hz)))/1000000,
					total_packet_count
					);

			last_display_time = now_and_then;
			display_time = now_and_then+local_cpu_hz;
			packet_count = 0;
		}

		// ``sleep'' until later. (sleep_time_cycles+start_time)
		//
		// Currently BUSY-WAITING; may want nanosleep for very low rate xmissions.
		// See calls in ../forger/forger.c for how to do this.
		now_and_then = start_time+(idx+1)*sleep_time_cycles;
		while (get_timestamp()<now_and_then)
			;
	}
	now_and_then=get_timestamp();

	// 20020202- Oops, was getting 1% errors because forgot the 'leftover' stuff during the last second.
	total_packet_count += packet_count;
	printf("Observed historical rate %9.3f, %lld packets over %0.3f seconds.\n",
			((double)((total_packet_count*(ABSOLUTE_MESSAGE_SIZE)*8)/(((double)(now_and_then-base_start_time))/local_cpu_hz)))/1000000,
			total_packet_count,
			(((double)(now_and_then-base_start_time))/local_cpu_hz)
			);
}


// ============================================================================
// Receive packets and just throw them away.
// This is only required so that polite linux flooder host will keep sending
// packets (otherwise an out-of-band failure packet is sent back to the client
// which causes an error in the sendto because the target socket is not valid).
// ============================================================================
void flood_recv(int passive) {
	int length;
	// Timing issues
	unsigned long long start_time, stop_time; // units in cycles

	// Hack: may want to avoid actally doing the recvfrom;
	//   minimizes the work that the server does-- in case it's also running tcpdump
	//   sill actually has an open socket-- so the client's send succeeds
	if (verbosity>0) {
		printf("Running %s...\n", passive ? "passively" : "NOT passively" );
	}
	if (passive) {
		sleep(365*24*60*60); // a year in seconds.
	}

	// loop for messages
	start_time = get_timestamp();
	packet_count=0;
	while (1) {
		// server will block on the recieve forever.
		if (!(length=full_recvfrom(connfd, msg, MAX_MESSAGE_SIZE, FLAG_BLOCK, &sock_addr, "server main input"))) {
			// 'soft' errors.
			continue;
		}
		packet_count++;

		stop_time = get_timestamp();
		if (verbosity>0 && (stop_time - start_time) > (unsigned long long)local_cpu_hz) {
			printf("Observed rate %9.3f over %ld packets\n", ((double)((packet_count*(ABSOLUTE_MESSAGE_SIZE)*8)/(((double)(stop_time-start_time))/local_cpu_hz)))/1000000, packet_count);
			start_time = get_timestamp();
			packet_count=0;
		}
	}
}


// ============================================================================
// Explain command-line syntax and die.
// ============================================================================
void usage(char *error) {
	if (error) { printf("\nUSAGE ERROR: %s\n", error); }
	printf("USAGE:\n");
	printf("./flooder -s <-v[vv]> <-b=[bufsize]> <-p>\n");
	printf("    -s: run a server (receives packets)\n");
	printf("    -v[vv]: run verbosely (-v) more verbosely (-vv) or in debug mode (-vvv)\n");
	printf("    -b=[bufsize]: set the socket buffer size to this value.\n"); 
	printf("    -p: run passively, (open socket, but do not actually read packets)\n\n");
	printf("./flooder -c <-v[vv]> <-b=[bufsize]> [rate] [duration] [server]\n");
	printf("    -c: run client (sends packets)\n");
	printf("    -v[vv]: run verbosely (-v) more verbosely (-vv) or in debug mode (-vvv)\n");
	printf("    -b=[bufsize]: set the socket buffer size to this value.\n"); 
	printf("    [rate]: an integer; the megabits per second at which to send\n");
	printf("    [duration]: an integer; how many seconds to maintain that rate\n");
	printf("    [server]: a hostname; the server to which we send packets.\n\n");
	printf("NOTE: ORDER OF ARGUMENTS MUST BE AS GIVEN!\n");
	flooder_shutdown(0);
}


// ============================================================================
// Parse the command line parameters into globals and the given locals.
// ============================================================================
void parse_command_line(int argc, char *argv[], int *send_duration, int *send_rate, int *passive) {
	if (argc<2) {
		usage("Too few arguments");
	}

	// get mode first.
	if (argv[1][1]=='c' || argv[1][1]=='C') {
		mode=FLOODER_MODE_CLIENT;
	} else if (argv[1][1]=='s' || argv[1][1]=='S') {
		mode=FLOODER_MODE_SERVER;
	} else {
		usage("First argument must be C or S (client or server)");
	}
	argv=&(argv[2]);
	argc-=2;
	
	// then get verbosity
	if (argc>0 && argv[0][0]=='-' && (argv[0][1]=='v' || argv[0][1]=='V')) {
		verbosity=strlen(argv[0])-1;
		argv = &(argv[1]);
		argc--;
	} else {
		verbosity=0;
	}

	// then get buffer size
	if (argc>0 && argv[0][0]=='-' && (argv[0][1]=='b' || argv[0][1]=='B') && argv[0][2]=='=') {
		if (sscanf(&(argv[0][3]), "%d", &bufsize)!=1) {
			usage("Failed scan of bufsize string!");
		}
		if (bufsize<=0) {
			usage("Scanned bad (negative or zero) bufsize string!");
		}

		argv = &(argv[1]);
		argc--;
	} else {
		bufsize=0;
	}

	if (mode==FLOODER_MODE_CLIENT) {
		if (argc!=3) {
			usage("Too few arguments for client");
		}
		sscanf(argv[0], "%d", send_rate);
		sscanf(argv[1], "%d", send_duration);
		hostname = argv[2];
	} else {
		if (argc==1) {
			*passive=1;
		} else if (argc!=0) {
			usage("Bad argument count for server");
		} else {
			*passive=0;
		}
	}
}


// ============================================================================
// The main routine.
// ============================================================================
int main(int argc, char *argv[]) {
	// timing issues
	int send_duration;    // units in seconds
	int send_rate;        // units in Mbps
	int passive;          // server sleeps with open socket.

	// parse command line.
	parse_command_line(argc, argv, &send_duration, &send_rate, &passive);

	// start up.
	if (!flooder_startup(send_duration, send_rate)) {
		flooder_shutdown(1);
	}

	// then do some work.
	switch(mode) {
		case FLOODER_MODE_CLIENT:
			// send a couple of packets to the server to link up, then do the real test.
			if (verbosity>0) {
				printf("Beginning send at rate %d for %d seconds\n", send_rate, send_duration);
			}
			flood_send(send_rate, send_duration);
			if (verbosity>0) {
				printf("Sending complete\n");
			}
			break;
		case FLOODER_MODE_SERVER:
			if (verbosity>0) {
				printf("Awaiting messages from client.\n");
			}
			flood_recv(passive);
			break;
	}

	// now we can quit!
	flooder_shutdown(0);
	return 0;
}

