/*
###############################################################################
# This file is part of TICKET, the Traffic Information Collection Kernel with
# Exact Timing. This program and all associated files are Copyright (C) 2001
# by Eric H. Weigle of Los Alamos National Laboratory.
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# ------------------------------------------------------------
# 
# You may contact the author (Eric H. Weigle) via email at ehw@lanl.gov,
# or via postal mail at MS D451, P.O. Box 1663, Los Alamos NM  87545.
# 
# Should you fail to reach the author at these addresses, please refer to the
# following websites for updated information or other means by which he can
# be contacted:
# 	http://public.lanl.gov/ehw/ 
# 	http://public.lanl.gov/radiant/
#
###############################################################################
*/
// #############################################################################
// Kernel networking header
// #############################################################################

#ifndef EW_TICKET_NETWORK_H
#define EW_TICKET_NETWORK_H

#include <linux/netdevice.h>  // network device structure
#include <linux/inetdevice.h> // for devinet_ioctl()
#include <linux/in.h>         // sockaddr_in
#include <linux/skbuff.h>     // socket buffers
#include <linux/if_ether.h>   // Defines ETH_ALEN, ETH_HLEN, ETH_DATA_LEN, ETH_FRAME_LEN, and struct ethhdr.
#include <linux/ip.h>         // Defines struct iphdr
#include <linux/udp.h>        // Defines struct udphdr
#include <linux/tcp.h>        // Defines struct tcphdr
#include <asm/checksum.h>     // checksums
#include <linux/delay.h>      // mdelay
#include "./common/shared_network.h"

// Ethernet interfaces in this box.
#define ETHERNET_IF_COUNT 8
#define ETHERNET_IF_INITIALIZER {NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL}
extern struct net_device *ew_eth[ETHERNET_IF_COUNT];

// How many packets have been dropped.
extern unsigned long ticket_drops;

// =============================================================================
// STRUCTURES
// =============================================================================

// These are stored in NBO
// Makes passing parameters around easier.
struct address_pair {
	unsigned char mac_addr[ETH_ALEN];
	unsigned int ip_addr;
};
// TODO struct address_pairs {
// TODO 	unsigned char dst_hw[ETH_ALEN];
// TODO 	unsigned char src_hw[ETH_ALEN];
// TODO 	unsigned int dst_ip;
// TODO 	unsigned int src_ip;
// TODO };


// =============================================================================
// FUNCTIONS
// =============================================================================
// bring up all interfaces, returns 0 on failure.
int ifupall(void);

// configure the devices for each mode.
// return number of output interfaces.
int configure_dump_dev(struct address_pair *ap, char **collector_in, char **collector_out, char **dumper);
void configure_route_dev(char *if0, char *if1);

// packet reception function before configuration.
// It just drops everything on the floor.
int netif_rx_drop(struct sk_buff *skb);

#endif

