/*
###############################################################################
# This file is part of TICKET, the Traffic Information Collection Kernel with
# Exact Timing. This program and all associated files are Copyright (C) 2001
# by Eric H. Weigle of Los Alamos National Laboratory.
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# ------------------------------------------------------------
# 
# You may contact the author (Eric H. Weigle) via email at ehw@lanl.gov,
# or via postal mail at MS D451, P.O. Box 1663, Los Alamos NM  87545.
# 
# Should you fail to reach the author at these addresses, please refer to the
# following websites for updated information or other means by which he can
# be contacted:
# 	http://public.lanl.gov/ehw/ 
# 	http://public.lanl.gov/radiant/
#
###############################################################################
*/
// #############################################################################
// Anonymization code's header.
//
// Just a structure; see also the dump structures (common/shared_dump.h)
// #############################################################################

#ifndef EW_ANON_H
#define EW_ANON_H

#include <time.h>      // ctime, timespec

// =============================================================================
// The stuff we've got after done mucking with it.
// =============================================================================
struct processed_headers {
	// Basic information.
	struct timespec timestamp; // time packet received.
	unsigned short length;     // 'off-the-wire' length (in bytes, up to 1514<2048=2^11)

	// Lowest (ethernet) level info
	unsigned int mac_dst;  // ethernet
	unsigned int mac_src;  // ethernet
	unsigned short toslen; // MAC TOS/802.3 length (usually 0x0800==IP)

	// IP level info.
	// Rest of data is only valid if toslen == 0x0800.
	unsigned int ip_src;      // IP
	unsigned int ip_dst;      // IP
	unsigned short ip_len;    // IP packet length
	unsigned char ip_hdr_len; // IP header length
	unsigned char protocol;   // IP protocol field

	// TCP/UDP level information.
	// Rest of data is only valid if protocol==6 (TCP) || protocol==17 (UDP)
	unsigned short src_port;  // TCP/UDP source port
	unsigned short dst_port;  // TCP/UDP destination port
	unsigned short upper_len; // TCP/UDP length field (in bytes)
} __attribute__ ((packed));

#endif

